/*
 Nokia 3120 KSS (Keypad SMS Sender)
 Version: 1.1
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 26/12/2011
 Last Modified: 29/01/2012

 */
 
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

#ifndef boolean
	typedef uint8_t boolean;
#endif 
#ifndef byte
	typedef uint8_t byte;
#endif

#ifndef NULL
#define NULL ((void *)0)
#endif

#define LOW 0
#define HIGH 1
#define OFF 0
#define ON 1
#define false 0
#define true 1

// Watchdog timeouts
#define T16MS 0
#define T32MS 1
#define T64MS 2
#define T128MS 3
#define T250MS 4
#define T500MS 5
#define T1S 6
#define T2S 7
#define T4S 8
#define T8S 9

// Defines which input line to enable on the darlington array and which transistor to activate to ground 
// E.g 52 = Enable output 5 (PA4) in darlington ULN2803A array and enable output 2 (PB1) in darlington ULN2003A array
#define buttonSelect 11
#define button2 31 // abc
#define button3 41 // def
#define button1 51
#define buttonUp 12
#define buttonDial 22
#define button6 42 // mno
#define button4 52 // ghi
#define buttonExit 13
#define button0 33
#define buttonHash 43
#define buttonStar 53
#define buttonDown 14
#define buttonHangup 24
#define button8 34 //tuv
#define button9 44 //wxyz
#define button7 54 //pqrs
#define buttonLeft 64
#define buttonRight 74
#define button5 84 //jkl

#define buttonPower PB4

#define buttonTopush 0
#define buttonPushes 1
#define numbersPushedonce 1

// Letter to button presses array
// { Button to push, Number of times to push }
byte letterTobutton[26][2] = {
{button2, 1}, // A (press button2 one time)
{button2, 2}, // B etc
{button2, 3}, // C
{button3, 1}, // D
{button3, 2}, // E
{button3, 3}, // F
{button4, 1}, // G
{button4, 2}, // H
{button4, 3}, // I
{button5, 1}, // J
{button5, 2}, // K
{button5, 3}, // L
{button6, 1}, // M
{button6, 2}, // N
{button6, 3}, // O
{button7, 1}, // P
{button7, 2}, // Q
{button7, 3}, // R
{button7, 4}, // S
{button8, 1}, // T
{button8, 2}, // U
{button8, 3}, // V
{button9, 1}, // W
{button9, 2}, // X
{button9, 3}, // Y
{button9, 4}, // Z
};

// Number to button presses array
byte numberTobutton[10][2] = {
{button0, 2}, // 0
{button1, 7}, // 1
{button2, 4}, // 2
{button3, 4}, // 3
{button4, 4}, // 4
{button5, 4}, // 5
{button6, 4}, // 6
{button7, 5}, // 7
{button8, 4}, // 8
{button9, 5}, // 9
};

// Sleep function
void system_sleep(void) {
	cbi(ADCSRA,ADEN); // Switch Analog to Digital converter Off
	set_sleep_mode(SLEEP_MODE_PWR_DOWN); // Set sleep mode
	sleep_mode(); // System sleeps here
	sbi(ADCSRA,ADEN);  // Switch Analog to Digital converter On
}

// Watchdog timeout values
// 0=16ms, 1=32ms, 2=64ms, 3=128ms, 4=250ms, 5=500ms, 6=1sec, 7=2sec, 8=4sec, 9=8sec
// Used from http://interface.khm.de/index.php/lab/experiments/sleep_watchdog_battery/
void setup_watchdog(int ii) {
	byte bb;
	if (ii > 9 ) ii=9;
	bb=ii & 7;
	if (ii > 7) bb|= (1<<5);
	bb|= (1<<WDCE);
	
	MCUSR &= ~(1<<WDRF);
	// Start timed sequence
	WDTCR |= (1<<WDCE) | (1<<WDE);
	// Set new watchdog timeout value
	WDTCR = bb;
	WDTCR |= _BV(WDIE);
}

// Turn off the Watchdog
void turnOffwatchdog(void) {
	cli(); // Turn interrupts off
	MCUSR &= ~(1<<WDRF);
	WDTCR |= (1<<WDCE) | (1<<WDE);
	WDTCR = 0;
	sei(); // Turn interrupts back on
}

// Watchdog sleep function - combines all the above functions into one
void watchdogSleep(int timeout){ 
	setup_watchdog(timeout);
	system_sleep();
	turnOffwatchdog();
}

// Setup
void setup(void) {
	sei(); // Turn on interrupts
}
